@@ -91,6 +91,11 @@ class TestCLexerNoErrors(unittest.TestCase):
     def test_special_names(self):
         self.assertTokensTypes('sizeof offsetof', ['SIZEOF', 'OFFSETOF'])
 
+    def test_new_keywords(self):
+        self.assertTokensTypes('_Bool', ['_BOOL'])
+        self.assertTokensTypes('_Atomic', ['_ATOMIC'])
+        self.assertTokensTypes('_Alignas _Alignof', ['_ALIGNAS', '_ALIGNOF'])
+
     def test_floating_constants(self):
         self.assertTokensTypes('1.5f', ['FLOAT_CONST'])
         self.assertTokensTypes('01.5', ['FLOAT_CONST'])
@@ -117,6 +122,9 @@ class TestCLexerNoErrors(unittest.TestCase):
     def test_char_constants(self):
         self.assertTokensTypes(r"""'x'""", ['CHAR_CONST'])
         self.assertTokensTypes(r"""L'x'""", ['WCHAR_CONST'])
+        self.assertTokensTypes(r"""u8'x'""", ['U8CHAR_CONST'])
+        self.assertTokensTypes(r"""u'x'""", ['U16CHAR_CONST'])
+        self.assertTokensTypes(r"""U'x'""", ['U32CHAR_CONST'])
         self.assertTokensTypes(r"""'\t'""", ['CHAR_CONST'])
         self.assertTokensTypes(r"""'\''""", ['CHAR_CONST'])
         self.assertTokensTypes(r"""'\?'""", ['CHAR_CONST'])
@@ -142,6 +150,9 @@ class TestCLexerNoErrors(unittest.TestCase):
     def test_string_literal(self):
         self.assertTokensTypes('"a string"', ['STRING_LITERAL'])
         self.assertTokensTypes('L"ing"', ['WSTRING_LITERAL'])
+        self.assertTokensTypes('u8"ing"', ['U8STRING_LITERAL'])
+        self.assertTokensTypes('u"ing"', ['U16STRING_LITERAL'])
+        self.assertTokensTypes('U"ing"', ['U32STRING_LITERAL'])
         self.assertTokensTypes(
             '"i am a string too \t"',
             ['STRING_LITERAL'])
@@ -167,10 +178,10 @@ class TestCLexerNoErrors(unittest.TestCase):
             ['STRING_LITERAL'])
         # The lexer is permissive and allows decimal escapes (not just octal)
         self.assertTokensTypes(
-            '"jx\9"',
+            r'"jx\9"',
             ['STRING_LITERAL'])
         self.assertTokensTypes(
-            '"fo\9999999"',
+            r'"fo\9999999"',
             ['STRING_LITERAL'])
 
     def test_mess(self):
@@ -346,6 +357,7 @@ class TestCLexerNoErrors(unittest.TestCase):
         #pragma "string"
         #pragma somestring="some_other_string"
         #pragma id 124124 and numbers 0235495
+        _Pragma("something else")
         59
         '''
         # Check that pragmas are tokenized, including trailing string
@@ -378,9 +390,19 @@ class TestCLexerNoErrors(unittest.TestCase):
             tb = self.clex.token()
             self.assertEqual(tb.type, 'PPPRAGMASTR')
 
-        t6 = self.clex.token()
-        self.assertEqual(t6.type, 'INT_CONST_DEC')
-        self.assertEqual(t6.lineno, 12)
+        t6a = self.clex.token()
+        t6l = self.clex.token()
+        t6b = self.clex.token()
+        t6r = self.clex.token()
+        self.assertEqual(t6a.type, '_PRAGMA')
+        self.assertEqual(t6l.type, 'LPAREN')
+        self.assertEqual(t6b.type, 'STRING_LITERAL')
+        self.assertEqual(t6b.value, '"something else"')
+        self.assertEqual(t6r.type, 'RPAREN')
+
+        t7 = self.clex.token()
+        self.assertEqual(t7.type, 'INT_CONST_DEC')
+        self.assertEqual(t7.lineno, 13)
 
 
 
@@ -452,7 +474,7 @@ class TestCLexerErrors(unittest.TestCase):
         self.assertLexerError("'", ERR_UNMATCHED_QUOTE)
         self.assertLexerError("'b\n", ERR_UNMATCHED_QUOTE)
         self.assertLexerError("'\\xaa\n'", ERR_UNMATCHED_QUOTE)
-        
+
         self.assertLexerError(r"'123\12a'", ERR_INVALID_CCONST)
         self.assertLexerError(r"'123\xabg'", ERR_INVALID_CCONST)
         self.assertLexerError("''", ERR_INVALID_CCONST)
